@@ -167,3 +167,56 @@ pub fn py_any_to_dynamic_value(value: &Bound<PyAny>) -> PyResult<DynamicValue> {
 
     Err(PyValueError::new_err("Unsupported value type"))
 }
+
+pub fn call_py_function_with_no_args(func_name: &str, func_opt: &Option<PyObject>) {
+    if func_opt.is_none() {
+        println!("{}: No function to call for {}", TAG, func_name);
+        return;
+    }
+
+    Python::with_gil(|py| {
+        let func = match func_opt.as_ref() {
+            Some(f) => f,
+            None => {
+                println!("{}: Function is None for {}", TAG, func_name);
+                return;
+            }
+        };
+        let result = func.call(py, (), None);
+        if let Err(e) = result {
+            println!(
+                "{}: Failed to call OutputLoggerProvider.{}: {:?}",
+                TAG, func_name, e
+            );
+        }
+    });
+}
+
+pub fn call_py_function_with_two_args(
+    func_name: &str,
+    func_opt: &Option<PyObject>,
+    arg1: &str,
+    arg2: &str,
+) {
+    if func_opt.is_none() {
+        println!("{}: No function to call for {}", TAG, func_name);
+        return;
+    }
+
+    Python::with_gil(|py| {
+        let func = match func_opt.as_ref() {
+            Some(f) => f,
+            None => {
+                println!("{}: Function is None for {}", TAG, func_name);
+                return;
+            }
+        };
+        let result = func.call(py, (arg1, arg2), None);
+        if let Err(e) = result {
+            println!(
+                "{}: Failed to call OutputLoggerProvider.{}: {:?}",
+                TAG, func_name, e
+            );
+        }
+    });
+}
