@@ -13,6 +13,7 @@ use crate::evaluation::evaluator_context::EvaluatorContext;
 use crate::evaluation::evaluator_value::{EvaluatorValue, EvaluatorValueType};
 use crate::evaluation::get_unit_id::get_unit_id;
 use crate::evaluation::ua_parser::UserAgentParser;
+use crate::event_logging::exposable_string;
 use crate::specs_response::spec_types::{Condition, Rule, Spec};
 use crate::{dyn_value, log_e, unwrap_or_return, StatsigErr};
 
@@ -24,8 +25,6 @@ pub struct Evaluator;
 
 lazy_static! {
     static ref EMPTY_STR: String = String::new();
-    static ref DEFAULT_RULE: String = "default".to_string();
-    static ref DISABLED_RULE: String = "disabled".to_string();
     static ref EMPTY_EVALUATOR_VALUE: EvaluatorValue =
         EvaluatorValue::new(EvaluatorValueType::Null);
     static ref EMPTY_DYNAMIC_VALUE: DynamicValue = DynamicValue::new();
@@ -80,7 +79,8 @@ impl Evaluator {
             SpecType::DynamicConfig => ctx.spec_store_data.values.dynamic_configs.get(spec_name),
             SpecType::Experiment => ctx.spec_store_data.values.dynamic_configs.get(spec_name),
             SpecType::Layer => ctx.spec_store_data.values.layer_configs.get(spec_name),
-        };
+        }
+        .map(|a| a.spec.as_ref());
 
         if try_apply_override(ctx, spec_name, spec_type, opt_spec) {
             return Ok(Recognition::Recognized);
@@ -155,8 +155,8 @@ impl Evaluator {
         ctx.result.bool_value = spec.default_value.get_bool() == Some(true);
         ctx.result.json_value = spec.default_value.get_json();
         ctx.result.rule_id = match spec.enabled {
-            true => Some(&DEFAULT_RULE),
-            false => Some(&DISABLED_RULE),
+            true => Some(&exposable_string::DEFAULT_RULE),
+            false => Some(&exposable_string::DISABLED_RULE),
         };
         ctx.finalize_evaluation(spec, None);
 
@@ -217,7 +217,7 @@ fn try_apply_config_mapping(
             ctx.reset_result();
             let pass = evaluate_pass_percentage(ctx, rule, spec_salt);
             if pass {
-                ctx.result.override_config_name = Some(mapping.new_config_name.clone());
+                ctx.result.override_config_name = Some(&mapping.new_config_name);
                 match Evaluator::evaluate(ctx, mapping.new_config_name.as_str(), spec_type) {
                     Ok(Recognition::Recognized) => {
                         return true;
@@ -246,21 +246,18 @@ fn try_apply_override(
     };
 
     match spec_type {
-        SpecType::Gate => adapter.get_gate_override(ctx.user.user_data, spec_name, &mut ctx.result),
+        SpecType::Gate => adapter.get_gate_override(ctx.user.user_ref, spec_name, &mut ctx.result),
 
         SpecType::DynamicConfig => {
-            adapter.get_dynamic_config_override(ctx.user.user_data, spec_name, &mut ctx.result)
+            adapter.get_dynamic_config_override(ctx.user.user_ref, spec_name, &mut ctx.result)
         }
 
-        SpecType::Experiment => adapter.get_experiment_override(
-            ctx.user.user_data,
-            spec_name,
-            &mut ctx.result,
-            opt_spec,
-        ),
+        SpecType::Experiment => {
+            adapter.get_experiment_override(ctx.user.user_ref, spec_name, &mut ctx.result, opt_spec)
+        }
 
         SpecType::Layer => {
-            adapter.get_layer_override(ctx.user.user_data, spec_name, &mut ctx.result)
+            adapter.get_layer_override(ctx.user.user_ref, spec_name, &mut ctx.result)
         }
     }
 }
@@ -471,7 +468,10 @@ fn evaluate_nested_gate<'a>(
         let expo = SecondaryExposure {
             gate: gate_name.clone(),
             gate_value: res.bool_value.to_string(),
-            rule_id: res.rule_id.unwrap_or(&EMPTY_STR).clone(),
+            rule_id: res
+                .rule_id
+                .unwrap_or(&exposable_string::EMPTY_STRING)
+                .clone(),
         };
 
         if res.sampling_rate.is_none() {
@@ -506,7 +506,7 @@ fn evaluate_config_delegate<'a>(
         return Ok(false);
     }
 
-    ctx.result.explicit_parameters = delegate_spec.explicit_parameters.as_ref();
+    ctx.result.explicit_parameters = delegate_spec.spec.explicit_parameters.as_ref();
     ctx.result.config_delegate = rule.config_delegate.as_ref();
 
     Ok(true)
@@ -521,7 +521,7 @@ fn evaluate_pass_percentage(ctx: &mut EvaluatorContext, rule: &Rule, spec_salt:
         return false;
     }
 
-    let rule_salt = rule.salt.as_ref().unwrap_or(&rule.id);
+    let rule_salt = rule.salt.as_deref().unwrap_or(rule.id.as_str());
     let unit_id = get_unit_id(ctx, &rule.id_type);
     let input = format!("{spec_salt}.{rule_salt}.{unit_id}");
     match ctx.hashing.evaluation_hash(&input) {
