@@ -5,14 +5,16 @@ use serde::{Deserialize, Serialize};
 use serde_json::Value;
 
 use crate::evaluation::dynamic_string::DynamicString;
+use crate::evaluation::evaluation_types::BaseEvaluation;
+use crate::event_logging::event_logger::EventLogger;
+use crate::event_logging::exposable_string::ExposableString;
 use crate::{
     evaluation::evaluation_types::{ExperimentEvaluation, LayerEvaluation},
-    event_logging::event_logger::EventLogger,
     statsig_type_factories::{extract_from_experiment_evaluation, make_layer},
     statsig_types::{Experiment, Layer},
     unwrap_or_return,
     user::StatsigUserInternal,
-    EvaluationDetails, SamplingProcessor, SecondaryExposure,
+    EvaluationDetails, SecondaryExposure,
 };
 
 pub type UserPersistedValues = HashMap<String, StickyValues>;
@@ -39,7 +41,7 @@ pub fn get_persistent_storage_key(user: &StatsigUserInternal, id_type: &String)
 pub struct StickyValues {
     pub value: bool,
     pub json_value: Option<HashMap<String, Value>>,
-    pub rule_id: Option<String>,
+    pub rule_id: Option<ExposableString>,
     pub group_name: Option<String>,
     pub secondary_exposures: Vec<SecondaryExposure>,
     pub undelegated_secondary_exposures: Option<Vec<SecondaryExposure>>,
@@ -55,7 +57,6 @@ pub fn make_layer_from_sticky_value(
     evaluation: LayerEvaluation,
     sticky_value: StickyValues,
     event_logger_ptr: Option<Weak<EventLogger>>,
-    sampling_processor: Option<Weak<SamplingProcessor>>,
     disable_exposure: bool,
 ) -> Layer {
     let details = EvaluationDetails {
@@ -63,21 +64,21 @@ pub fn make_layer_from_sticky_value(
         lcut: sticky_value.time,
         received_at: Some(Utc::now().timestamp_millis() as u64),
     };
+
     make_layer(
         user.to_loggable(),
         name,
         Some(evaluation),
         details,
         event_logger_ptr,
-        sticky_value.config_version,
         disable_exposure,
-        sampling_processor,
-        None,
     )
 }
 
 pub fn make_sticky_value_from_layer(layer: &Layer) -> Option<StickyValues> {
     let layer_evaluation = unwrap_or_return!(layer.__evaluation.as_ref(), None);
+    let config_version = extract_config_version(&layer_evaluation.base);
+
     Some(StickyValues {
         value: true,
         json_value: Some(layer_evaluation.value.clone()),
@@ -88,7 +89,7 @@ pub fn make_sticky_value_from_layer(layer: &Layer) -> Option<StickyValues> {
         config_delegate: layer_evaluation.allocated_experiment_name.clone(),
         explicit_parameters: Some(layer_evaluation.explicit_parameters.clone()),
         time: layer.details.lcut,
-        config_version: layer.__version,
+        config_version,
     })
 }
 
@@ -108,18 +109,18 @@ pub fn make_experiment_from_sticky_value(
     Experiment {
         name,
         value,
-        rule_id,
+        rule_id: rule_id.unperformant_to_string(),
         id_type,
         group_name,
         details,
         __evaluation: maybe_evaluation,
-        __version: sticky_value.config_version,
-        __override_config_name: None,
     }
 }
 
 pub fn make_sticky_value_from_experiment(experiment: &Experiment) -> Option<StickyValues> {
     let experiment_evaluation = unwrap_or_return!(&experiment.__evaluation, None);
+    let config_version = extract_config_version(&experiment_evaluation.base);
+
     Some(StickyValues {
         value: true, // For sticky value, if it's being saved, it should always be true
         json_value: Some(experiment_evaluation.value.clone()),
@@ -132,6 +133,13 @@ pub fn make_sticky_value_from_experiment(experiment: &Experiment) -> Option<Stic
         config_delegate: None,
         explicit_parameters: experiment_evaluation.explicit_parameters.clone(),
         time: experiment.details.lcut,
-        config_version: experiment.__version,
+        config_version,
     })
 }
+
+fn extract_config_version(evaluation: &BaseEvaluation) -> Option<u32> {
+    evaluation
+        .exposure_info
+        .as_ref()
+        .and_then(|info| info.version)
+}
