@@ -1,16 +1,20 @@
 use crate::evaluation::dynamic_string::DynamicString;
 use crate::evaluation::{dynamic_returnable::DynamicReturnable, evaluator_value::EvaluatorValue};
+use crate::event_logging::exposable_string::ExposableString;
 use crate::DynamicValue;
 use ahash::HashMap as AHashMap;
 use serde::{Deserialize, Deserializer, Serialize};
+use serde_with::skip_serializing_none;
 use std::collections::HashMap;
 
 use super::condition_key::ConditionKey;
+use super::spec_directory::SpecDirectory;
 use super::{cmab_types::CMABConfig, param_store_types::ParameterStore};
 
 // DO_NOT_CLONE: Please do not add the Clone trait to this struct. We intentionally
 // avoid cloning this data at all costs as it can be quite large.
 
+#[skip_serializing_none]
 #[derive(Serialize, Deserialize, PartialEq, Debug)] /* DO_NOT_CLONE */
 #[serde(rename_all = "camelCase")]
 pub struct Spec {
@@ -32,13 +36,14 @@ pub struct Spec {
     pub fields_used: Option<Vec<String>>,
 }
 
+#[skip_serializing_none]
 #[derive(Serialize, Deserialize, PartialEq, Debug)] /* DO_NOT_CLONE */
 #[serde(rename_all = "camelCase")]
 pub struct Rule {
     pub name: String,
     pub pass_percentage: f64,
     pub return_value: DynamicReturnable,
-    pub id: String,
+    pub id: ExposableString,
     pub salt: Option<String>,
     pub conditions: Vec<ConditionKey>,
     pub id_type: DynamicString,
@@ -60,23 +65,27 @@ pub struct Condition {
     pub id_type: DynamicString,
 }
 
+#[skip_serializing_none]
 #[derive(Serialize, Deserialize, PartialEq, Debug)] /* DO_NOT_CLONE */
 pub struct OverrideRule {
     pub rule_name: String,
     pub start_time: Option<i64>,
 }
 
+#[skip_serializing_none]
 #[derive(Serialize, Deserialize, PartialEq, Debug)] /* DO_NOT_CLONE */
 pub struct ConfigMapping {
     pub new_config_name: String,
     pub rules: Vec<OverrideRule>,
 }
 
+#[skip_serializing_none]
 #[derive(Serialize, Deserialize, PartialEq, Debug, Default)] /* DO_NOT_CLONE */
 pub struct SpecsResponseFull {
-    pub feature_gates: AHashMap<String, Spec>,
-    pub dynamic_configs: AHashMap<String, Spec>,
-    pub layer_configs: AHashMap<String, Spec>,
+    pub company_id: Option<String>,
+    pub feature_gates: SpecDirectory,
+    pub dynamic_configs: SpecDirectory,
+    pub layer_configs: SpecDirectory,
     pub condition_map: AHashMap<ConditionKey, Condition>,
     pub experiment_to_layer: HashMap<String, String>,
     pub has_updates: bool,
@@ -92,8 +101,11 @@ pub struct SpecsResponseFull {
     pub cmab_configs: Option<HashMap<String, CMABConfig>>,
     pub overrides: Option<HashMap<String, Vec<ConfigMapping>>>,
     pub override_rules: Option<HashMap<String, Rule>>,
+    pub id_lists: Option<HashMap<String, bool>>,
+    pub response_format: Option<String>,
 }
 
+#[skip_serializing_none]
 #[derive(Deserialize)]
 pub struct SpecsResponseNoUpdates {
     pub has_updates: bool,
